# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cicadad/protos/datastore.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cicadad/protos/datastore.proto',
  package='datastore',
  syntax='proto3',
  serialized_options=b'Z-github.com/cicadatesting/datastore-client/api',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1e\x63icadad/protos/datastore.proto\x12\tdatastore\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\"6\n\x14\x41\x64\x64UserResultRequest\x12\x0e\n\x06userID\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x0c\"\xae\x01\n\x18SetScenarioResultRequest\x12\x12\n\nscenarioID\x18\x01 \x01(\t\x12,\n\x06output\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12/\n\texception\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x0c\n\x04logs\x18\x04 \x01(\t\x12\x11\n\ttimeTaken\x18\x05 \x01(\x01\")\n\x16MoveUserResultsRequest\x12\x0f\n\x07userIDs\x18\x01 \x03(\t\"*\n\x17MoveUserResultsResponse\x12\x0f\n\x07results\x18\x01 \x03(\x0c\"/\n\x19MoveScenarioResultRequest\x12\x12\n\nscenarioID\x18\x01 \x01(\t\"\xbb\x01\n\x1aMoveScenarioResultResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12,\n\x06output\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12/\n\texception\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x0c\n\x04logs\x18\x04 \x01(\t\x12\x11\n\ttimestamp\x18\x05 \x01(\t\x12\x11\n\ttimeTaken\x18\x06 \x01(\x01\"6\n\x15\x44istributeWorkRequest\x12\x0c\n\x04work\x18\x01 \x01(\x05\x12\x0f\n\x07userIDs\x18\x02 \x03(\t\"$\n\x12GetUserWorkRequest\x12\x0e\n\x06userID\x18\x01 \x01(\t\"#\n\x13GetUserWorkResponse\x12\x0c\n\x04work\x18\x01 \x01(\x05\x32\xfe\x03\n\tDatastore\x12H\n\rAddUserResult\x12\x1f.datastore.AddUserResultRequest\x1a\x16.google.protobuf.Empty\x12P\n\x11SetScenarioResult\x12#.datastore.SetScenarioResultRequest\x1a\x16.google.protobuf.Empty\x12X\n\x0fMoveUserResults\x12!.datastore.MoveUserResultsRequest\x1a\".datastore.MoveUserResultsResponse\x12\x61\n\x12MoveScenarioResult\x12$.datastore.MoveScenarioResultRequest\x1a%.datastore.MoveScenarioResultResponse\x12J\n\x0e\x44istributeWork\x12 .datastore.DistributeWorkRequest\x1a\x16.google.protobuf.Empty\x12L\n\x0bGetUserWork\x12\x1d.datastore.GetUserWorkRequest\x1a\x1e.datastore.GetUserWorkResponseB/Z-github.com/cicadatesting/datastore-client/apib\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])




_ADDUSERRESULTREQUEST = _descriptor.Descriptor(
  name='AddUserResultRequest',
  full_name='datastore.AddUserResultRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='userID', full_name='datastore.AddUserResultRequest.userID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='datastore.AddUserResultRequest.result', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=160,
)


_SETSCENARIORESULTREQUEST = _descriptor.Descriptor(
  name='SetScenarioResultRequest',
  full_name='datastore.SetScenarioResultRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='scenarioID', full_name='datastore.SetScenarioResultRequest.scenarioID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output', full_name='datastore.SetScenarioResultRequest.output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exception', full_name='datastore.SetScenarioResultRequest.exception', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='logs', full_name='datastore.SetScenarioResultRequest.logs', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeTaken', full_name='datastore.SetScenarioResultRequest.timeTaken', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=163,
  serialized_end=337,
)


_MOVEUSERRESULTSREQUEST = _descriptor.Descriptor(
  name='MoveUserResultsRequest',
  full_name='datastore.MoveUserResultsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='userIDs', full_name='datastore.MoveUserResultsRequest.userIDs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=339,
  serialized_end=380,
)


_MOVEUSERRESULTSRESPONSE = _descriptor.Descriptor(
  name='MoveUserResultsResponse',
  full_name='datastore.MoveUserResultsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='datastore.MoveUserResultsResponse.results', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=382,
  serialized_end=424,
)


_MOVESCENARIORESULTREQUEST = _descriptor.Descriptor(
  name='MoveScenarioResultRequest',
  full_name='datastore.MoveScenarioResultRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='scenarioID', full_name='datastore.MoveScenarioResultRequest.scenarioID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=426,
  serialized_end=473,
)


_MOVESCENARIORESULTRESPONSE = _descriptor.Descriptor(
  name='MoveScenarioResultResponse',
  full_name='datastore.MoveScenarioResultResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='datastore.MoveScenarioResultResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output', full_name='datastore.MoveScenarioResultResponse.output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exception', full_name='datastore.MoveScenarioResultResponse.exception', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='logs', full_name='datastore.MoveScenarioResultResponse.logs', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='datastore.MoveScenarioResultResponse.timestamp', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeTaken', full_name='datastore.MoveScenarioResultResponse.timeTaken', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=476,
  serialized_end=663,
)


_DISTRIBUTEWORKREQUEST = _descriptor.Descriptor(
  name='DistributeWorkRequest',
  full_name='datastore.DistributeWorkRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='work', full_name='datastore.DistributeWorkRequest.work', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userIDs', full_name='datastore.DistributeWorkRequest.userIDs', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=665,
  serialized_end=719,
)


_GETUSERWORKREQUEST = _descriptor.Descriptor(
  name='GetUserWorkRequest',
  full_name='datastore.GetUserWorkRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='userID', full_name='datastore.GetUserWorkRequest.userID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=721,
  serialized_end=757,
)


_GETUSERWORKRESPONSE = _descriptor.Descriptor(
  name='GetUserWorkResponse',
  full_name='datastore.GetUserWorkResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='work', full_name='datastore.GetUserWorkResponse.work', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=794,
)

_SETSCENARIORESULTREQUEST.fields_by_name['output'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_SETSCENARIORESULTREQUEST.fields_by_name['exception'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_MOVESCENARIORESULTRESPONSE.fields_by_name['output'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_MOVESCENARIORESULTRESPONSE.fields_by_name['exception'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
DESCRIPTOR.message_types_by_name['AddUserResultRequest'] = _ADDUSERRESULTREQUEST
DESCRIPTOR.message_types_by_name['SetScenarioResultRequest'] = _SETSCENARIORESULTREQUEST
DESCRIPTOR.message_types_by_name['MoveUserResultsRequest'] = _MOVEUSERRESULTSREQUEST
DESCRIPTOR.message_types_by_name['MoveUserResultsResponse'] = _MOVEUSERRESULTSRESPONSE
DESCRIPTOR.message_types_by_name['MoveScenarioResultRequest'] = _MOVESCENARIORESULTREQUEST
DESCRIPTOR.message_types_by_name['MoveScenarioResultResponse'] = _MOVESCENARIORESULTRESPONSE
DESCRIPTOR.message_types_by_name['DistributeWorkRequest'] = _DISTRIBUTEWORKREQUEST
DESCRIPTOR.message_types_by_name['GetUserWorkRequest'] = _GETUSERWORKREQUEST
DESCRIPTOR.message_types_by_name['GetUserWorkResponse'] = _GETUSERWORKRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AddUserResultRequest = _reflection.GeneratedProtocolMessageType('AddUserResultRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDUSERRESULTREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.AddUserResultRequest)
  })
_sym_db.RegisterMessage(AddUserResultRequest)

SetScenarioResultRequest = _reflection.GeneratedProtocolMessageType('SetScenarioResultRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETSCENARIORESULTREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.SetScenarioResultRequest)
  })
_sym_db.RegisterMessage(SetScenarioResultRequest)

MoveUserResultsRequest = _reflection.GeneratedProtocolMessageType('MoveUserResultsRequest', (_message.Message,), {
  'DESCRIPTOR' : _MOVEUSERRESULTSREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.MoveUserResultsRequest)
  })
_sym_db.RegisterMessage(MoveUserResultsRequest)

MoveUserResultsResponse = _reflection.GeneratedProtocolMessageType('MoveUserResultsResponse', (_message.Message,), {
  'DESCRIPTOR' : _MOVEUSERRESULTSRESPONSE,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.MoveUserResultsResponse)
  })
_sym_db.RegisterMessage(MoveUserResultsResponse)

MoveScenarioResultRequest = _reflection.GeneratedProtocolMessageType('MoveScenarioResultRequest', (_message.Message,), {
  'DESCRIPTOR' : _MOVESCENARIORESULTREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.MoveScenarioResultRequest)
  })
_sym_db.RegisterMessage(MoveScenarioResultRequest)

MoveScenarioResultResponse = _reflection.GeneratedProtocolMessageType('MoveScenarioResultResponse', (_message.Message,), {
  'DESCRIPTOR' : _MOVESCENARIORESULTRESPONSE,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.MoveScenarioResultResponse)
  })
_sym_db.RegisterMessage(MoveScenarioResultResponse)

DistributeWorkRequest = _reflection.GeneratedProtocolMessageType('DistributeWorkRequest', (_message.Message,), {
  'DESCRIPTOR' : _DISTRIBUTEWORKREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.DistributeWorkRequest)
  })
_sym_db.RegisterMessage(DistributeWorkRequest)

GetUserWorkRequest = _reflection.GeneratedProtocolMessageType('GetUserWorkRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERWORKREQUEST,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.GetUserWorkRequest)
  })
_sym_db.RegisterMessage(GetUserWorkRequest)

GetUserWorkResponse = _reflection.GeneratedProtocolMessageType('GetUserWorkResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERWORKRESPONSE,
  '__module__' : 'cicadad.protos.datastore_pb2'
  # @@protoc_insertion_point(class_scope:datastore.GetUserWorkResponse)
  })
_sym_db.RegisterMessage(GetUserWorkResponse)


DESCRIPTOR._options = None

_DATASTORE = _descriptor.ServiceDescriptor(
  name='Datastore',
  full_name='datastore.Datastore',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=797,
  serialized_end=1307,
  methods=[
  _descriptor.MethodDescriptor(
    name='AddUserResult',
    full_name='datastore.Datastore.AddUserResult',
    index=0,
    containing_service=None,
    input_type=_ADDUSERRESULTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetScenarioResult',
    full_name='datastore.Datastore.SetScenarioResult',
    index=1,
    containing_service=None,
    input_type=_SETSCENARIORESULTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='MoveUserResults',
    full_name='datastore.Datastore.MoveUserResults',
    index=2,
    containing_service=None,
    input_type=_MOVEUSERRESULTSREQUEST,
    output_type=_MOVEUSERRESULTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='MoveScenarioResult',
    full_name='datastore.Datastore.MoveScenarioResult',
    index=3,
    containing_service=None,
    input_type=_MOVESCENARIORESULTREQUEST,
    output_type=_MOVESCENARIORESULTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DistributeWork',
    full_name='datastore.Datastore.DistributeWork',
    index=4,
    containing_service=None,
    input_type=_DISTRIBUTEWORKREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUserWork',
    full_name='datastore.Datastore.GetUserWork',
    index=5,
    containing_service=None,
    input_type=_GETUSERWORKREQUEST,
    output_type=_GETUSERWORKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DATASTORE)

DESCRIPTOR.services_by_name['Datastore'] = _DATASTORE

# @@protoc_insertion_point(module_scope)
