# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cicadad/protos/container_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cicadad/protos/container_service.proto',
  package='container_service',
  syntax='proto3',
  serialized_options=b'Z.github.com/cicadatesting/container-service/api',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n&cicadad/protos/container_service.proto\x12\x11\x63ontainer_service\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\xb8\x03\n\x15StartContainerRequest\x12\r\n\x05image\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x03 \x03(\t\x12\x0e\n\x06labels\x18\x04 \x03(\t\x12>\n\x03\x65nv\x18\x05 \x03(\x0b\x32\x31.container_service.StartContainerRequest.EnvEntry\x12\x30\n\x07volumes\x18\x06 \x03(\x0b\x32\x1f.container_service.DockerVolume\x12-\n\x08hostPort\x18\x07 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12\x32\n\rcontainerPort\x18\x08 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12-\n\x07network\x18\t \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x31\n\rcreateNetwork\x18\n \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x1a*\n\x08\x45nvEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"3\n\x0c\x44ockerVolume\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65stination\x18\x02 \x01(\t\"+\n\x14StopContainerRequest\x12\x13\n\x0b\x63ontainerID\x18\x01 \x01(\t2\xb8\x01\n\x10\x43ontainerService\x12R\n\x0eStartContainer\x12(.container_service.StartContainerRequest\x1a\x16.google.protobuf.Empty\x12P\n\rStopContainer\x12\'.container_service.StopContainerRequest\x1a\x16.google.protobuf.EmptyB0Z.github.com/cicadatesting/container-service/apib\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])




_STARTCONTAINERREQUEST_ENVENTRY = _descriptor.Descriptor(
  name='EnvEntry',
  full_name='container_service.StartContainerRequest.EnvEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='container_service.StartContainerRequest.EnvEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='container_service.StartContainerRequest.EnvEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=521,
  serialized_end=563,
)

_STARTCONTAINERREQUEST = _descriptor.Descriptor(
  name='StartContainerRequest',
  full_name='container_service.StartContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='container_service.StartContainerRequest.image', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='container_service.StartContainerRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='command', full_name='container_service.StartContainerRequest.command', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='container_service.StartContainerRequest.labels', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='env', full_name='container_service.StartContainerRequest.env', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='volumes', full_name='container_service.StartContainerRequest.volumes', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hostPort', full_name='container_service.StartContainerRequest.hostPort', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='containerPort', full_name='container_service.StartContainerRequest.containerPort', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='network', full_name='container_service.StartContainerRequest.network', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='createNetwork', full_name='container_service.StartContainerRequest.createNetwork', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STARTCONTAINERREQUEST_ENVENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=123,
  serialized_end=563,
)


_DOCKERVOLUME = _descriptor.Descriptor(
  name='DockerVolume',
  full_name='container_service.DockerVolume',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='container_service.DockerVolume.source', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination', full_name='container_service.DockerVolume.destination', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=565,
  serialized_end=616,
)


_STOPCONTAINERREQUEST = _descriptor.Descriptor(
  name='StopContainerRequest',
  full_name='container_service.StopContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='containerID', full_name='container_service.StopContainerRequest.containerID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=618,
  serialized_end=661,
)

_STARTCONTAINERREQUEST_ENVENTRY.containing_type = _STARTCONTAINERREQUEST
_STARTCONTAINERREQUEST.fields_by_name['env'].message_type = _STARTCONTAINERREQUEST_ENVENTRY
_STARTCONTAINERREQUEST.fields_by_name['volumes'].message_type = _DOCKERVOLUME
_STARTCONTAINERREQUEST.fields_by_name['hostPort'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT32VALUE
_STARTCONTAINERREQUEST.fields_by_name['containerPort'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT32VALUE
_STARTCONTAINERREQUEST.fields_by_name['network'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_STARTCONTAINERREQUEST.fields_by_name['createNetwork'].message_type = google_dot_protobuf_dot_wrappers__pb2._BOOLVALUE
DESCRIPTOR.message_types_by_name['StartContainerRequest'] = _STARTCONTAINERREQUEST
DESCRIPTOR.message_types_by_name['DockerVolume'] = _DOCKERVOLUME
DESCRIPTOR.message_types_by_name['StopContainerRequest'] = _STOPCONTAINERREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StartContainerRequest = _reflection.GeneratedProtocolMessageType('StartContainerRequest', (_message.Message,), {

  'EnvEntry' : _reflection.GeneratedProtocolMessageType('EnvEntry', (_message.Message,), {
    'DESCRIPTOR' : _STARTCONTAINERREQUEST_ENVENTRY,
    '__module__' : 'cicadad.protos.container_service_pb2'
    # @@protoc_insertion_point(class_scope:container_service.StartContainerRequest.EnvEntry)
    })
  ,
  'DESCRIPTOR' : _STARTCONTAINERREQUEST,
  '__module__' : 'cicadad.protos.container_service_pb2'
  # @@protoc_insertion_point(class_scope:container_service.StartContainerRequest)
  })
_sym_db.RegisterMessage(StartContainerRequest)
_sym_db.RegisterMessage(StartContainerRequest.EnvEntry)

DockerVolume = _reflection.GeneratedProtocolMessageType('DockerVolume', (_message.Message,), {
  'DESCRIPTOR' : _DOCKERVOLUME,
  '__module__' : 'cicadad.protos.container_service_pb2'
  # @@protoc_insertion_point(class_scope:container_service.DockerVolume)
  })
_sym_db.RegisterMessage(DockerVolume)

StopContainerRequest = _reflection.GeneratedProtocolMessageType('StopContainerRequest', (_message.Message,), {
  'DESCRIPTOR' : _STOPCONTAINERREQUEST,
  '__module__' : 'cicadad.protos.container_service_pb2'
  # @@protoc_insertion_point(class_scope:container_service.StopContainerRequest)
  })
_sym_db.RegisterMessage(StopContainerRequest)


DESCRIPTOR._options = None
_STARTCONTAINERREQUEST_ENVENTRY._options = None

_CONTAINERSERVICE = _descriptor.ServiceDescriptor(
  name='ContainerService',
  full_name='container_service.ContainerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=664,
  serialized_end=848,
  methods=[
  _descriptor.MethodDescriptor(
    name='StartContainer',
    full_name='container_service.ContainerService.StartContainer',
    index=0,
    containing_service=None,
    input_type=_STARTCONTAINERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopContainer',
    full_name='container_service.ContainerService.StopContainer',
    index=1,
    containing_service=None,
    input_type=_STOPCONTAINERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CONTAINERSERVICE)

DESCRIPTOR.services_by_name['ContainerService'] = _CONTAINERSERVICE

# @@protoc_insertion_point(module_scope)
